require "rubygems"
gem "haml", ">= 3.2"
require File.expand_path("../html2haml/version", __FILE__)
require "haml/util"
require "haml/parser"
require "haml/error"
require "html2haml/html"

module Html2haml
end
